import pytest
from decouple import config
from redis import Redis


@pytest.fixture(scope="session")
def redis_client_fixture(request):
    """
    This fixture establishes a connection to Redis, and yields address and port of that connection
    """

    redis = Redis(
        host=config("REDIS_HOST", default="127.0.0.1", cast=str),
        port=config("REDIS_PORT", default=6379, cast=int),
        db=config("REDIS_DATABASE", default=0, cast=int),
        decode_responses=True,
        socket_timeout=30,
        socket_connect_timeout=30,
        max_connections=7500,
    )

    # Teardown code
    def teardown():
        # Clean up resources or perform any necessary cleanup
        redis.flushdb()
        redis.close()

    # Register the teardown function to be executed after the test session
    request.addfinalizer(teardown)

    connection_pool = redis.connection_pool

    # Access the address and port
    address = connection_pool.connection_kwargs["host"]
    port = connection_pool.connection_kwargs["port"]

    yield address, port


@pytest.fixture()
def translate_client_fixture(live_server, redis_client_fixture):
    """
    Fixture for creating melon-translate-client instance
    """
    from melon_translate_client import Client

    translate_url = live_server.url
    port = translate_url.split(":")[2]
    redis_address, redis_port = redis_client_fixture

    translate_client = Client(
        service_address=config("TRANSLATE_ADDRESS", default="http://localhost"),
        service_port=port,
        cache_address=redis_address,
        cache_port=redis_port,
    )

    yield translate_client



import pytest


@pytest.mark.django_db
def test_redis_client_fixture(redis_client_fixture):
    redis_address, redis_port = redis_client_fixture

    assert redis_address == "127.0.0.1"
    assert redis_port == 6379


@pytest.mark.django_db
def test_translate_client_fixture(translate_client_fixture):
    client = translate_client_fixture

    assert client
    assert client.address == "http://localhost"
    assert client.port
    assert client.cache_address == "127.0.0.1"



"""Melon Translate client"""

# from redis_om import Migrator

from ._client import Client

__all__ = ["Client"]

#
# class _InitializeModule:
#     __instance = None
#
#     def __new__(cls, *args, **kwargs):
#         if not cls.__instance:
#             cls.__instance = super().__new__(cls, *args, **kwargs)
#         return cls.__instance
#
#     def __init__(self):
#         self.setup_lock()
#         self.setup_models()
#
#     def setup_lock(self):
#         pass
#
#     @staticmethod
#     def setup_models():
#         # redis = get_redis_connection()
#         Migrator().run()
#
#
# _ = _InitializeModule()



from typing import List, Optional

from redis_om import Field, HashModel


class TranslationKey(HashModel):
    snake_name: str = Field(index=True)
    id_name: str = Field(index=True)
    uuid: str = Field(index=True)
    views: Optional[str] = Field(
        index=True
    )  # this need to stay optional for now because po translations don't have assigned view
    occurrences: List[str]
    usage_context: Optional[str]
    flags: Optional[str]
    category: int = 0  # default value is 0
    translation: str
    language: str


class Views(HashModel):
    name: str = Field(index=True)
    language: str
    updated_at: int


class Occurrences(HashModel):
    name: str = Field(index=True)
    language: str
    updated_at: int
